!vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv!
!  "CO2 capture with solid sorbent: CFD model of an" &            !
!  "innovative reactor  concept"    @2016                         !
!   ***L. Barelli, G. Bidini, F. Gallorini                        !
!                                                                 !
!   "Synthesis and test of sorbents based on calcium "&           !
!   "aluminates for SE-SR"     @2014                              !
!    ***L. Barelli, G. Bidini, F. Gallorini                       !
!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^!     
 
   SUBROUTINE USR_RATES_DES(NP, pM, IJK, DES_RATES)

      USE compar
      USE constant
      USE des_thermo
      USE des_rxns
      USE discretelement
      USE energy
      USE fldvar
      USE funits
      USE geometry
      USE indices
      USE param
      USE param1
      USE physprop
      USE rxns
      USE run
      USE usr
      USE fun_avg
      USE functions

      USE parallel
      USE sendrecv
      USE toleranc



      IMPLICIT NONE

      INTEGER, INTENT(IN) :: NP  ! Global index of particle
      INTEGER, INTENT(IN) :: pM  ! Solid phase index of particle NP
      INTEGER, INTENT(IN) :: IJK ! Fluid cell index containing NP

      INTEGER  LB,UB

! Calculated reaction rates. (reacted moles per sec)
      DOUBLE PRECISION, INTENT(OUT) :: DES_RATES(NO_OF_DES_RXNS)


! Reaction specific variables:
!`````````````````````````````````````````````````````````````````````//
!峣J/mol*k
     DOUBLE PRECISION :: R_constant=8.3d-3
! Solids Diameter (m)
      DOUBLE PRECISION :: Dp0
! Bounded phase temperatures (K)
      DOUBLE PRECISION xTg   ! gas phase
      DOUBLE PRECISION xTs   ! solids phase
! Gas phase pressure in KPa
      DOUBLE PRECISION Pg_KPa
! Gas phase concentrations (mol/m^3)
      DOUBLE PRECISION c_CO2
!CaCO3(s)
      DOUBLE PRECISION :: c_CaCO3
! Partial pressures (KPa)
      DOUBLE PRECISION p_CO2
!Ӧco2ƽѹ
      DOUBLE PRECISION :: p_CO2_e
! Total solids surface area per volume (1/m)
      DOUBLE PRECISION Sa
! Initial adsorption capacity for Cao (g)
      DOUBLE PRECISION :: W0_Cao=26.91
! The weight of the adsorbed of CO2 (g)
      DOUBLE PRECISION :: Wads_CO2=16
!  molar weight of CaO (g/mol)
      DOUBLE PRECISION :: M_CaO=56
! molar weight of CO2 (g/mol)
      DOUBLE PRECISION :: M_CO2=44
!CO2 capture capacity of the sorbent (gCO2/gads 100)
      DOUBLE PRECISION Ct_CO2
! carbonation conversion of C
      DOUBLE PRECISION A_C
! Apparent first order heterogeneous reation rate constant  (m/sec)
      DOUBLE PRECISION k_so
!ֽⷴӦʣm/sec
     DOUBLE PRECISION :: k_s_CaCO3
! Temporary values for reversible reactions
      DOUBLE PRECISION FWD, RVS, NET


! Prevent reactions when mass fraction falls below c_Limiter.
      DOUBLE PRECISION, parameter :: c_Limiter = 1.0d-7

!     Uncomment the following line when customizing this usr_rates_des.f to include generated file species.inc:
!
     INCLUDE 'species.inc'


! Reaction rates:
!`````````````````````````````````````````````````````````````````````//
! Include reaction rates here. Reaction rates should be stored in the
! variable DES_RATES. The reaction name given in the data file can be
! used to store the rate in the appropriate array location. Additional
! input format parameters are given in Section 4.11 of the code Readme.

! Calculate particle diameter.
      Dp0 = 2.0d0 * DES_RADIUS(NP)

! Particle surface area (m^2)
      Sa = Pi* Dp0 * Dp0   ! (m^2)

! Calculate the bounded temperatures.
      xTg = max(min(T_g(IJK),   TMAX), 650.0d0)
      xTs = max(min(DES_T_s(NP), TMAX), 650.0d0)

! Convert gas pressure to KPa
      Pg_KPa = 1.0d-3 * P_g(IJK)  !  KPa

! Initialize partial pressures and concentrations.
      c_CO2  = ZERO;   p_CO2 = ZERO;
      
! Calculate co2 capture capacity of CO2
   !   IF(X_g(IJK,CO2) > c_Limiter) then
       !  Ct_CO2=(Wads_CO2/W0_CO2)*100
     ! ENDIF

! Calculate conversion rate of C
       IF(X_g(IJK,CO2) > c_Limiter) then
         if (W0_cao .ne. 0 .and. M_CO2 .ne. 0) then
            A_C=(Wads_CO2/W0_cao) * (M_CaO/M_CO2)
         else
           write (*,*) "W0_cao=", W0_cao,"M_CO2=", M_CO2
          ENDIF
     ENDIF
     

      

!**********************************************************************!
!                        Heterogeneous Reactions                       !
!**********************************************************************!
      IF(X_g(IJK,CO2) > c_Limiter) THEN


! CaO + CO2 --> CaCO3  (kg-mole/m^3.s)
!---------------------------------------------------------------------//
! Apparent first order heterogeneous reation rate constant  (m/sec)
         k_so = 7.97d-10 * exp(-1.58/(R_constant*xTs))

! Reaction rate:
         DES_RATES(R1)  = k_so*(1-(0.55/A_C))
        
         write (*,*) " DES_RATES(R1)=", DES_RATES(R1)
      ELSE

         DES_RATES(R1)  = ZERO
         write (*,*) " DES_RATES(R1)=", DES_RATES(R1)

      ENDIF



      ! IF((des_X_s(NP,CaCO3))>c_Limiter) then
	  !c_CaCO3 = ROP_s(IJK,1)*X_s(IJK,1,CaCO3)/MW_s(1,CaCO3)
	  !write(*,*) 'c_CACO3', c_CACO3
	  !ENDIF
	  
	  IF(X_g(IJK,CO2) > c_Limiter) then ! С ֵc_Limiter Ϊû
               p_CO2 = Pg_KPa * X_g(IJK,CO2) * (MW_MIX_g(IJK) / MW_g(CO2))
               c_CO2 = RO_g(IJK) * X_g(IJK,CO2) / Mw_g(CO2)
			   write(*,*)  'p_CO2', p_CO2, 'c_CO2', c_CO2
      ENDIF
      
      LB=NO_OF_RXNS+1
      UB=NO_OF_RXNS+NO_OF_DES_RXNS
      ReactionRates(IJK,LB)=ReactionRates(IJK,LB)+DES_RATES(R1)
      RETURN

      END SUBROUTINE USR_RATES_DES
